#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import re

INFINITY = float("inf")
""" Infinity value alternative to math module infinity
compatible with Python versions 2 and 3  """

ITERABLES = (list, tuple)
""" The tuple that defined the various base types
that are considered to be generally "iterable" """

MOBILE_REGEX = re.compile(
    r"(android|bb\\d+|meego).+mobile|avantgo|bada\\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino",
    re.I | re.M,
)  # @UndefinedVariable
""" The regular expression that is going to be used
to validate the complete user agent string for mobile """

TABLET_REGEX = re.compile(
    r"(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino|android|ipad|playbook|silk",
    re.I | re.M,
)  # @UndefinedVariable
""" The regular expression that is going to be used
to validate the complete user agent string for tablet """

MOBILE_PREFIX_REGEX = re.compile(
    r"1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\\-(n|u)|c55\\/|capi|ccwa|cdm\\-|cell|chtm|cldc|cmd\\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\\-s|devi|dica|dmob|do(c|p)o|ds(12|\\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\\-|_)|g1 u|g560|gene|gf\\-5|g\\-mo|go(\\.w|od)|gr(ad|un)|haie|hcit|hd\\-(m|p|t)|hei\\-|hi(pt|ta)|hp( i|ip)|hs\\-c|ht(c(\\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\\-(20|go|ma)|i230|iac( |\\-|\\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\\/)|klon|kpt |kwc\\-|kyo(c|k)|le(no|xi)|lg( g|\\/(k|l|u)|50|54|\\-[a-w])|libw|lynx|m1\\-w|m3ga|m50\\/|ma(te|ui|xo)|mc(01|21|ca)|m\\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\\-2|po(ck|rt|se)|prox|psio|pt\\-g|qa\\-a|qc(07|12|21|32|60|\\-[2-7]|i\\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\\-|oo|p\\-)|sdk\\/|se(c(\\-|0|1)|47|mc|nd|ri)|sgh\\-|shar|sie(\\-|m)|sk\\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\\-|v\\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\\-|tdg\\-|tel(i|m)|tim\\-|t\\-mo|to(pl|sh)|ts(70|m\\-|m3|m5)|tx\\-9|up(\\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\\-|your|zeto|zte\\-",
    re.I | re.M,
)  # @UndefinedVariable
""" The regular expression to test the base prefix
of the user agent string for mobile browser """

BODY_REGEX = re.compile(r"<body[^<>]*?>(.*?)</body>", re.I)  # @UndefinedVariable
""" Regular expression that is going to be used in the matching
of the partial contents (child) nodes of a body HTML node """

TAG_REGEX = re.compile(r"<[^<]*?>")
""" The regular expression that is going to be used in the matching
of an html/xml based node in the HTML to plain text conversion """

EMAIL_REGEX = re.compile(
    r"((?P<name>^.+) \<(?P<email_a>[\w\d\._%+-]+@[\w\d\.\-]+)\>)|(?P<email_b>[\w\d\._%+-]+@[\w\d\.\-]+)$",
    flags=re.UNICODE,  # @UndefinedVariable
)
""" The regular expression that is going to be used
in the matching of email lines, it supports both the
extended (including name) and the simplified versions """

BROWSER_INFO = [
    dict(identity="Edge", sub_string="Edge"),
    dict(identity="Chrome", sub_string="Chrome"),
    dict(identity="Safari", sub_string="Safari"),
    dict(identity="Opera"),
    dict(identity="Firefox", sub_string="Firefox"),
    dict(identity="Explorer", sub_string="MSIE", version_search="MSIE "),
    dict(identity="AppleWebKit", sub_string="AppleWebKit"),
    dict(identity="Googlebot", sub_string="Googlebot", interactive=False, bot=True),
    dict(identity="Bingbot", sub_string="Bingbot", interactive=False, bot=True),
    dict(identity="DuckDuckBot", sub_string="DuckDuckBot", interactive=False, bot=True),
    dict(identity="netius", sub_string="netius", interactive=False),
]
""" List that contains the complete information used
for the parsing and identification of the browser information
from a typical user agent string """

OS_INFO = [
    dict(identity="Windows", sub_string="Win"),
    dict(identity="Mac", sub_string="Mac"),
    dict(identity="iPhone/iPod", sub_string="iPhone"),
    dict(sub_string="Linux", identity="Linux"),
]
""" List that contains the complete information used
for the parsing and identification of the os information
from a typical user agent string """

WINDOWS_LOCALE = dict(
    af="Afrikaans",
    sq="Albanian",
    am="Amharic",
    ar_DZ="Arabic_Algeria",
    ar_BH="Arabic_Bahrain",
    ar_EG="Arabic_Egypt",
    ar_IQ="Arabic_Iraq",
    ar_JO="Arabic_Jordan",
    ar_KW="Arabic_Kuwait",
    ar_LB="Arabic_Lebanon",
    ar_LY="Arabic_Libya",
    ar_MA="Arabic_Morocco",
    ar_OM="Arabic_Oman",
    ar_QA="Arabic_Qatar",
    ar_SA="Arabic_Saudi Arabia",
    ar_SY="Arabic_Syria",
    ar_TN="Arabic_Tunisia",
    ar_AE="Arabic_United Arab Emirates",
    ar_YE="Arabic_Yemen",
    hy="Armenian",
    az_AZ="Azeri_Cyrillic",
    eu="Basque",
    be="Belarusian",
    bn="Bengali_Bangladesh",
    bs="Bosnian",
    bg="Bulgarian",
    my="Burmese",
    ca="Catalan",
    zh_CN="Chinese_China",
    zh_HK="Chinese_Hong Kong SAR",
    zh_MO="Chinese_Macau SAR",
    zh_SG="Chinese_Singapore",
    zh_TW="Chinese_Taiwan",
    hr="Croatian",
    cs="Czech",
    da="Danish",
    dv="Divehi; Dhivehi; Maldivian",
    nl_BE="Dutch_Belgium",
    nl_NL="Dutch_Netherlands",
    en_AU="English_Australia",
    en_BZ="English_Belize",
    en_CA="English_Canada",
    en_CB="English_Caribbean",
    en_GB="English_Great Britain",
    en_IN="English_India",
    en_IE="English_Ireland",
    en_JM="English_Jamaica",
    en_NZ="English_New Zealand",
    en_PH="English_Phillippines",
    en_ZA="English_Southern Africa",
    en_TT="English_Trinidad",
    en_US="English_United States",
    et="Estonian",
    fo="Faroese",
    fa="Farsi_Persian",
    fi="Finnish",
    fr_BE="French_Belgium",
    fr_CA="French_Canada",
    fr_FR="French_France",
    fr_LU="French_Luxembourg",
    fr_CH="French_Switzerland",
    mk="FYRO Macedonia",
    gd_IE="Gaelic_Ireland",
    gd="Gaelic_Scotland",
    de_AT="German_Austria",
    de_DE="German_Germany",
    de_LI="German_Liechtenstein",
    de_LU="German_Luxembourg",
    de_CH="German_Switzerland",
    el="Greek",
    gn="Guarani_Paraguay",
    gu="Gujarati",
    he="Hebrew",
    hi="Hindi",
    hu="Hungarian",
    id="Indonesian",
    it_IT="Italian_Italy",
    it_CH="Italian_Switzerland",
    ja="Japanese",
    kn="Kannada",
    ks="Kashmiri",
    kk="Kazakh",
    km="Khmer",
    ko="Korean",
    lo="Lao",
    la="Latin",
    lv="Latvian",
    lt="Lithuanian",
    ms_BN="Malay_Brunei",
    ms_MY="Malay_Malaysia",
    ml="Malayalam",
    mt="Maltese",
    mi="Maori",
    mr="Marathi",
    mn="Mongolian",
    ne="Nepali",
    no_NO="Norwegian_Bokml",
    pl="Polish",
    pt_BR="Portuguese_Brazil",
    pt_PT="Portuguese_Portugal",
    pa="Punjabi",
    rm="Raeto-Romance",
    ro_MO="Romanian_Moldova",
    ro="Romanian_Romania",
    ru="Russian",
    ru_MO="Russian_Moldova",
    sa="Sanskrit",
    sr_SP="Serbian_Cyrillic",
    tn="Setsuana",
    sd="Sindhi",
    si="Sinhala; Sinhalese",
    sk="Slovak",
    sl="Slovenian",
    so="Somali",
    sb="Sorbian",
    es_AR="Spanish_Argentina",
    es_BO="Spanish_Bolivia",
    es_CL="Spanish_Chile",
    es_CO="Spanish_Colombia",
    es_CR="Spanish_Costa Rica",
    es_DO="Spanish_Dominican Republic",
    es_EC="Spanish_Ecuador",
    es_SV="Spanish_El Salvador",
    es_GT="Spanish_Guatemala",
    es_HN="Spanish_Honduras",
    es_MX="Spanish_Mexico",
    es_NI="Spanish_Nicaragua",
    es_PA="Spanish_Panama",
    es_PY="Spanish_Paraguay",
    es_PE="Spanish_Peru",
    es_PR="Spanish_Puerto Rico",
    es_ES="Spanish_Spain (Traditional)",
    es_UY="Spanish_Uruguay",
    es_VE="Spanish_Venezuela",
    sw="Swahili",
    sv_FI="Swedish_Finland",
    sv_SE="Swedish_Sweden",
    tg="Tajik",
    ta="Tamil",
    tt="Tatar",
    te="Telugu",
    th="Thai",
    bo="Tibetan",
    ts="Tsonga",
    tr="Turkish",
    tk="Turkmen",
    uk="Ukrainian",
    UTF_8="Unicode",
    ur="Urdu",
    uz_UZ="Uzbek_Cyrillic",
    vi="Vietnamese",
    cy="Welsh",
    xh="Xhosa",
    yi="Yiddish",
    zu="Zulu",
)
""" The map containing the associations between the
normalized version/descriptor of the locale and the
longer windows version of them so that it may be used
when setting locales for windows based operative systems """

SLUG_PERMUTATIONS = (
    ("á", "a"),
    ("é", "e"),
    ("í", "i"),
    ("ó", "o"),
    ("ú", "u"),
    ("à", "a"),
    ("ã", "a"),
    ("õ", "o"),
    ("â", "a"),
    ("ê", "e"),
    ("ô", "o"),
    ("ç", "c"),
    ("ä", "a"),
    ("ë", "e"),
    ("ï", "i"),
    ("ö", "o"),
    ("ü", "u"),
    ("ÿ", "y"),
    ("æ", "ae"),
    ("œ", "oe"),
)
""" The complete set of permutation tuples to be used in
a slug based simplification process """



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier


class Person(appier.Model):
    identifier = appier.field(type=int, index=True, increment=True, default=True)

    identifier_safe = appier.field(type=int, index=True, increment=True, safe=True)

    name = appier.field()

    age = appier.field(type=int)

    info = appier.field(type=dict)

    father = appier.field(
        type=appier.reference("Person", name="identifier", dumpall=True)
    )

    brother = appier.field(type=appier.reference("Person", name="identifier"))

    car = appier.field(type=appier.reference("Car", name="identifier"), eager=True)

    cats = appier.field(type=appier.references("Cat", name="identifier"))

    @classmethod
    def validate(cls):
        return super(Person, cls).validate() + [
            appier.not_null("name"),
            appier.not_empty("name"),
            appier.not_duplicate("name", cls._name()),
        ]


class Cat(appier.Model):
    identifier = appier.field(type=int, index=True, increment=True, default=True)

    identifier_safe = appier.field(type=int, index=True, increment=True, safe=True)

    name = appier.field()

    friend = appier.field(type=appier.reference("Cat", name="identifier"))


class Car(appier.Model):
    identifier = appier.field(type=int, index=True, increment=True, default=True)

    identifier_safe = appier.field(type=int, index=True, increment=True, safe=True)

    name = appier.field()

    brand = appier.field()

    variant = appier.field()

    garage = appier.field(
        type=appier.reference("Garage", name="identifier"), eager=True
    )


class Garage(appier.Model):
    identifier = appier.field(type=int, index=True, increment=True, default=True)

    identifier_safe = appier.field(type=int, index=True, increment=True, safe=True)

    name = appier.field()

    address = appier.field(
        type=appier.reference("Address", name="identifier"), eager=True
    )


class Address(appier.Model):
    identifier = appier.field(type=int, index=True, increment=True, default=True)

    identifier_safe = appier.field(type=int, index=True, increment=True, safe=True)

    street = appier.field()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import util
from . import common
from . import legacy
from . import exceptions


class Git(object):
    @classmethod
    def is_git(cls, path=None):
        path = path or common.base().get_base_path()
        try:
            result = util.execute(["git", "status"], path=path)
        except OSError:
            return False
        code = result["code"]
        return code == 0

    @classmethod
    def clone(cls, url, path=None, raise_e=True):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "clone", url], path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        return message

    @classmethod
    def fetch(cls, flags=[], path=None, raise_e=True):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "fetch"] + flags, path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        return message

    @classmethod
    def pull(cls, flags=[], path=None, raise_e=True):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "pull"] + flags, path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        return message

    @classmethod
    def push(cls, flags=[], path=None, raise_e=True):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "push"] + flags, path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        return message

    @classmethod
    def commit(cls, message="Update", flags=[], path=None, raise_e=True):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "commit", "-m", message] + flags, path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        return message

    @classmethod
    def checkout(cls, branch="master", flags=[], path=None, raise_e=True):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "checkout", branch] + flags, path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        return message

    @classmethod
    def add(cls, target="*", flags=[], path=None, raise_e=True):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "add", target] + flags, path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        return message

    @classmethod
    def add_upstream(cls, url, name="upstream", path=None, raise_e=True):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "remote", "add", name, url], path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        return message

    @classmethod
    def remove_upstream(cls, name="upstream", path=None, raise_e=True):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "remote", "remove", name], path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        return message

    @classmethod
    def config(cls, key, value, _global=True, path=None, raise_e=True):
        path = path or common.base().get_base_path()
        result = util.execute(
            ["git", "config", "--global" if _global else "", key, value], path=path
        )
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        return message

    @classmethod
    def get_config(cls, key, _global=True, path=None, raise_e=False):
        path = path or common.base().get_base_path()
        result = util.execute(
            ["git", "config", "--global" if _global else "", "--get", key], path=path
        )
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        value = message.strip()
        return value

    @classmethod
    def get_branches(cls, names=False, path=None, raise_e=False):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "branch"], path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        branches = message.strip()
        branches = branches.split("\n")
        branches = [(value.lstrip("* "), value.startswith("*")) for value in branches]
        if names:
            branches = [branch[0] for branch in branches]
        return branches

    @classmethod
    def get_branch(cls, path=None, raise_e=False):
        path = path or common.base().get_base_path()
        branches = cls.get_branches(path=path, raise_e=raise_e)
        for branch, selected in branches:
            if not selected:
                continue
            return branch
        return None

    @classmethod
    def get_commit(cls, path=None, raise_e=False):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "rev-parse", "HEAD"], path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        commit = message.strip()
        return commit

    @classmethod
    def get_origin(cls, path=None, raise_e=False):
        path = path or common.base().get_base_path()
        result = util.execute(
            ["git", "config", "--get", "remote.origin.url"], path=path
        )
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        origin = message.strip()
        origin = cls.safe_origin(origin)
        return origin

    @classmethod
    def get_repo_path(cls, path=None, raise_e=False):
        path = path or common.base().get_base_path()
        result = util.execute(["git", "rev-parse", "--show-toplevel"], path=path)
        if cls._wrap_error(result, raise_e=raise_e):
            return None
        message = result.get("stdout", "")
        repo_path = message.strip()
        return repo_path

    @classmethod
    def safe_origin(cls, origin, display_l=3):
        parse = legacy.urlparse(origin)
        safe_l = []
        if parse.scheme:
            safe_l.append(parse.scheme + "://")
        if parse.username:
            safe_l.append(parse.username)
        if parse.password:
            obfuscated = util.obfuscate(parse.password)
            safe_l.append(":" + obfuscated)
        if parse.username:
            safe_l.append("@")
        if parse.hostname:
            safe_l.append(parse.hostname)
        if parse.path:
            safe_l.append(parse.path)
        return "".join(safe_l)

    @classmethod
    def norm_origin(cls, origin, prefix="https://"):
        if origin.startswith(("http://", "https://")):
            return origin
        if origin.endswith(".git"):
            origin = origin[:-4]
        origin = origin.replace(":", "/")
        origin = prefix + origin
        return origin

    @classmethod
    def parse_origin(cls, origin, safe=True):
        parse = legacy.urlparse(origin)
        if safe and not parse.scheme:
            origin = cls.norm_origin(origin)
            return cls.parse_origin(origin, safe=False)
        scheme = parse.scheme if parse.scheme else "ssh"
        username = parse.username if parse.username else None
        password = parse.password if parse.password else None
        hostname = parse.hostname if parse.hostname else None
        path = parse.path if parse.path else None
        return dict(
            scheme=scheme,
            username=username,
            password=password,
            hostname=hostname,
            path=path,
        )

    @classmethod
    def _wrap_error(cls, result, raise_e=False):
        code = result["code"]
        if code == 0:
            return False
        if raise_e:
            raise exceptions.OperationalError(
                message=result.get("stderr", "") or result.get("stdout", "")
            )
        return True



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import meta
from . import legacy
from . import observer


class Controller(legacy.with_meta(meta.Indexed, observer.Observable)):
    """
    Top level abstract controller class from which all the
    concrete controller should inherit. Should provide structure
    for the creation of part of code that may act as entry levels
    for the business logic, these methods should be called action
    methods and may be implemented in the controller or application.

    Most of the controller logic is a simple redirection to the owner
    object that should be an application.
    """

    def __init__(self, owner, *args, **kwargs):
        observer.Observable.__init__(self, *args, **kwargs)
        self.owner = owner

    def __getattr__(self, name):
        if hasattr(self.owner, name):
            return getattr(self.owner, name)
        raise AttributeError("'%s' not found" % name)

    def register(self, lazy=False):
        if lazy:
            return
        self.setup()

    def unregister(self, lazy=False):
        if lazy:
            return
        self.teardown()

    def setup(self):
        pass

    def teardown(self):
        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import calendar
import datetime
import unittest

import appier


class CronSchedulerTest(unittest.TestCase):

    def test_basic(self):
        state = dict(value=0)

        def increment():
            state["value"] += 1

        scheduler = appier.CronScheduler(None)
        task = scheduler.schedule(
            lambda: increment(),
            "11",
            now=datetime.datetime(2013, 1, 1, hour=1, minute=1),
        )
        self.assertNotEqual(task, None)
        self.assertEqual(isinstance(task, appier.SchedulerTask), True)
        self.assertEqual(task.enabled, True)
        self.assertEqual(
            scheduler.next_run(), datetime.datetime(2013, 1, 1, hour=1, minute=11)
        )

        scheduler.tick(
            now_ts=calendar.timegm(
                datetime.datetime(2013, 1, 1, hour=1, minute=1).utctimetuple()
            )
        )
        self.assertEqual(state["value"], 0)
        self.assertEqual(scheduler.timeout, 600)

        scheduler.tick(
            now_ts=calendar.timegm(
                datetime.datetime(2013, 1, 1, hour=1, minute=11).utctimetuple()
            )
        )
        self.assertEqual(state["value"], 1)
        self.assertEqual(scheduler.timeout, 3600)

        scheduler.tick(
            now_ts=calendar.timegm(
                datetime.datetime(2013, 1, 1, hour=2, minute=11).utctimetuple()
            )
        )
        self.assertEqual(state["value"], 2)
        self.assertEqual(scheduler.timeout, 3600)

    def test_scheduler_date(self):
        state = dict(value=0)

        def increment():
            state["value"] += 1

        scheduler = appier.CronScheduler(None)
        task = scheduler.schedule(
            lambda: increment(),
            appier.SchedulerDate(minutes=11),
            now=datetime.datetime(2013, 1, 1, hour=1, minute=1),
        )
        self.assertNotEqual(task, None)
        self.assertEqual(isinstance(task, appier.SchedulerTask), True)
        self.assertEqual(task.enabled, True)
        self.assertEqual(
            scheduler.next_run(), datetime.datetime(2013, 1, 1, hour=1, minute=11)
        )

        scheduler.tick(
            now_ts=calendar.timegm(
                datetime.datetime(2013, 1, 1, hour=1, minute=1).utctimetuple()
            )
        )
        self.assertEqual(state["value"], 0)
        self.assertEqual(scheduler.timeout, 600)

        scheduler.tick(
            now_ts=calendar.timegm(
                datetime.datetime(2013, 1, 1, hour=1, minute=11).utctimetuple()
            )
        )
        self.assertEqual(state["value"], 1)
        self.assertEqual(scheduler.timeout, 3600)

    def test_week_days(self):
        state = dict(value=0)

        def increment():
            state["value"] += 1

        scheduler = appier.CronScheduler(None)
        task = scheduler.schedule(
            lambda: increment(),
            appier.SchedulerDate(minutes=11, days_of_month=10, days_of_week=2),
            now=datetime.datetime(2013, 1, 1, hour=1, minute=1),
        )
        self.assertNotEqual(task, None)
        self.assertEqual(isinstance(task, appier.SchedulerTask), True)
        self.assertEqual(task.enabled, True)
        self.assertEqual(
            scheduler.next_run(), datetime.datetime(2013, 4, 10, hour=0, minute=11)
        )


class SchedulerDateTest(unittest.TestCase):

    def test_from_cron(self):
        date = appier.SchedulerDate.from_cron("11")
        self.assertEqual(date.minutes, set((11,)))
        self.assertEqual(date.hours, set(range(0, 24)))
        self.assertEqual(date.days_of_month, set(range(1, 32)))
        self.assertEqual(date.months, set(range(1, 13)))
        self.assertEqual(date.days_of_week, set(range(0, 7)))

    def test_next_run(self):
        date = appier.SchedulerDate.from_cron("11")

        value = date.next_run(now=datetime.datetime(2013, 1, 1, hour=1, minute=1))
        self.assertEqual(value, datetime.datetime(2013, 1, 1, hour=1, minute=11))

        value = date.next_run(now=datetime.datetime(2013, 1, 1, hour=1, minute=12))
        self.assertEqual(value, datetime.datetime(2013, 1, 1, hour=2, minute=11))

        value = date.next_run(now=datetime.datetime(2013, 12, 31, hour=23, minute=12))
        self.assertEqual(value, datetime.datetime(2014, 1, 1, hour=0, minute=11))

    def test_next_run_complex(self):
        date = appier.SchedulerDate.from_cron("11 3 10,16,20")
        value = date.next_run(now=datetime.datetime(2013, 1, 1, hour=1, minute=1))
        self.assertEqual(value, datetime.datetime(2013, 1, 10, hour=3, minute=11))

        value = date.next_run(now=datetime.datetime(2013, 1, 10, hour=4, minute=1))
        self.assertEqual(value, datetime.datetime(2013, 1, 16, hour=3, minute=11))

        date = appier.SchedulerDate.from_cron("* 3 10,16,20")
        value = date.next_run(now=datetime.datetime(2013, 1, 10, hour=3, minute=1))
        self.assertEqual(value, datetime.datetime(2013, 1, 10, hour=3, minute=2))

        value = date.next_run(now=datetime.datetime(2013, 1, 10, hour=3, minute=2))
        self.assertEqual(value, datetime.datetime(2013, 1, 10, hour=3, minute=3))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import os
import time
import uuid
import hmac
import zlib
import pickle
import shelve
import base64
import hashlib
import datetime

from . import config
from . import legacy
from . import redisdb
from . import exceptions

EXPIRE_TIME = datetime.timedelta(days=31)
""" The default expire time to be used in new sessions
in case no expire time is provided to the creation of
the session instance """


class Session(object):
    """
    Abstract session class to be used as a reference in
    the implementation of the proper classes.

    Some of the global functionality may be abstracted
    under this class and reused in the concrete classes.
    """

    def __init__(
        self, name="session", expire=EXPIRE_TIME, sid=None, address=None, growing=True
    ):
        object.__init__(self)
        self.sid = sid if sid else self._gen_sid()
        self.name = name
        self.address = address
        self.create = time.time()
        self.modify = self.create
        self.duration = self._to_seconds(expire)
        self.expire = self.create + self.duration
        self.growing = growing
        self.dirty = True
        self.transient = False
        self.data_t = dict()

    def __len__(self):
        return self.data_t.__len__()

    def __getitem__(self, key):
        return self.data_t.__getitem__(key)

    def __setitem__(self, key, value):
        self.mark(extend=self.growing)
        self.data_t.__setitem__(key, value)

    def __delitem__(self, key):
        self.mark(extend=self.growing)
        self.data_t.__delitem__(key)

    def __iter__(self):
        return self.data_t.__iter__()

    def __contains__(self, item):
        return self.data_t.__contains__(item)

    def __nonzero__(self):
        return True

    def __bool__(self):
        return True

    def __getstate__(self):
        return dict(
            sid=self.sid,
            name=self.name,
            address=self.address,
            create=self.create,
            modify=self.modify,
            duration=self.duration,
            expire=self.expire,
            growing=self.growing,
            dirty=self.dirty,
        )

    def __setstate__(self, state):
        for name in (
            "sid",
            "name",
            "address",
            "create",
            "modify",
            "duration",
            "expire",
            "growing",
            "dirty",
        ):
            value = state.get(name, None)
            setattr(self, name, value)
        self.data_t = dict()

    @classmethod
    def new(cls, *args, **kwargs):
        return cls(*args, **kwargs)

    @classmethod
    def get_s(cls, sid, request=None):
        return cls()

    @classmethod
    def expire(cls, sid):
        pass

    @classmethod
    def count(cls):
        return 0

    @classmethod
    def all(cls):
        return {}

    @classmethod
    def open(cls):
        cls.gc()

    @classmethod
    def close(cls):
        pass

    @classmethod
    def empty(cls):
        pass

    @classmethod
    def gc(cls):
        pass

    def keys(self):
        return ()

    def iterkeys(self):
        return self.keys()

    def values(self):
        return ()

    def itervalues(self):
        return self.values()

    def items(self):
        return ()

    def iteritems(self):
        return self.items()

    def pop(self, key, default=None):
        if not key in self:
            return default
        value = self[key]
        del self[key]
        return value

    def start(self):
        pass

    def flush(self, request=None):
        self.mark(dirty=False)

    def mark(self, dirty=True, extend=False):
        self.dirty = dirty
        if extend:
            self.extend()

    def extend(self, duration=None):
        duration = duration or self.duration
        if not duration:
            return
        self.modify = time.time()
        self.duration = duration
        self.expire = self.modify + duration
        self.mark()
        return self.expire

    def is_expired(self):
        has_expire = hasattr(self, "expire")
        if not has_expire:
            return False

        current = time.time()
        return current >= self.expire

    def is_dirty(self):
        if not hasattr(self, "dirty"):
            return True
        return self.dirty

    def is_loaded(self):
        return True

    def ensure(self, *args, **kwargs):
        return self

    def get(self, key, default=None):
        try:
            value = self.__getitem__(key)
        except KeyError:
            value = default
        return value

    def set(self, key, value):
        self.__setitem__(key, value)

    def delete(self, key, force=False):
        if not force and not key in self:
            return
        self.__delitem__(key)

    def get_t(self, key, default=None):
        try:
            value = self.data_t[key]
        except KeyError:
            value = default
        return value

    def set_t(self, key, value):
        self.data_t[key] = value

    def delete_t(self, key, force=False):
        if not force and not key in self.data_t:
            return
        del self.data_t[key]

    def timeout(self):
        current = time.time()
        return self.expire - current

    def set_transient(self, value=True):
        """
        Marks the current session as transient only meaning
        that all of the operations will be made in memory
        and not persisted through session engine.

        This behaviour is ideal for secret key based session
        that do not have long lived life-cycles.

        :type value: bool
        :param value: The new value to be set for transient.
        """

        self.transient = value

    def _gen_sid(self):
        token_s = str(uuid.uuid4())
        token_s = legacy.bytes(token_s)
        token = hashlib.sha256(token_s).hexdigest()
        return token

    def _to_seconds(self, delta):
        return (
            delta.microseconds + (delta.seconds + delta.days * 24 * 3600) * 10**6
        ) / 10**6


class MockSession(Session):
    """
    Temporary mock session to be used while no final
    session is yet defined in the request, this is a
    special case of a session and should not comply
    with the pre-defined standard operations.
    """

    def __init__(self, request, name="mock", *args, **kwargs):
        Session.__init__(self, name=name, *args, **kwargs)
        self.request = request
        self.setter = None

    def __setitem__(self, key, value):
        if self.transient:
            return Session.__setitem__(self, key, value)
        session = self.ensure(sid=self.sid, address=self.address)
        return session.__setitem__(key, value)

    def __setstate__(self, state):
        Session.__setstate__(self, state)
        self.request = None

    def is_loaded(self):
        return False

    def ensure(self, *args, **kwargs):
        if self.transient:
            return self
        self._ensure_names(kwargs)
        session_c = self.request.session_c
        session = session_c.new(*args, **kwargs)
        self.request.session = session
        self.request.set_cookie = "sid=%s" % session.sid
        return session

    def _ensure_names(self, kwargs):
        for name in ("sid", "address"):
            if name in kwargs:
                continue
            kwargs[name] = getattr(self, name)


class DataSession(Session):
    def __init__(self, *args, **kwargs):
        Session.__init__(self, *args, **kwargs)
        self.data = {}

    def __len__(self):
        return Session.__len__(self) + self.data.__len__()

    def __getitem__(self, key):
        try:
            return self.data.__getitem__(key)
        except KeyError:
            return Session.__getitem__(self, key)

    def __setitem__(self, key, value):
        self.mark(extend=self.growing)
        return self.data.__setitem__(key, value)

    def __delitem__(self, key):
        try:
            self.mark(extend=self.growing)
            return self.data.__delitem__(key)
        except KeyError:
            return Session.__delitem__(self, key)

    def __iter__(self):
        return self._merged.__iter__()

    def __contains__(self, item):
        return Session.__contains__(self, item) or self.data.__contains__(item)

    def __getstate__(self):
        state = Session.__getstate__(self)
        state.update(data=self.data)
        return state

    def __setstate__(self, state):
        Session.__setstate__(self, state)
        for name in ("data",):
            setattr(self, name, state[name])

    def keys(self):
        return self._merged.keys()

    def values(self):
        return self._merged.values()

    def items(self):
        return self._merged.items()

    def sorted(self):
        keys = self.keys()
        keys = list(keys)
        keys.sort()
        return keys

    @property
    def _merged(self):
        if not self.data_t:
            return self.data
        if not self.data:
            return self.data_t
        merged = dict()
        merged.update(self.data_t)
        merged.update(self.data)
        return merged


class MemorySession(DataSession):
    SESSIONS = {}
    """ Global static sessions map where all the
    (in-memory) session instances are going to be
    stored to be latter retrieved """

    def __init__(self, name="session", *args, **kwargs):
        DataSession.__init__(self, name=name, *args, **kwargs)
        self["sid"] = self.sid

    @classmethod
    def new(cls, *args, **kwargs):
        session = cls(*args, **kwargs)
        cls.SESSIONS[session.sid] = session.__getstate__()
        return session

    @classmethod
    def get_s(cls, sid, request=None):
        state = cls.SESSIONS.get(sid, None)
        if not state:
            return state
        session = cls.__new__(cls)
        session.__setstate__(state)
        is_expired = session.is_expired()
        if is_expired:
            cls.expire(sid)
        session = None if is_expired else session
        return session

    @classmethod
    def expire(cls, sid):
        del cls.SESSIONS[sid]

    @classmethod
    def count(cls):
        return len(cls.SESSIONS)

    @classmethod
    def all(cls):
        return cls.SESSIONS

    @classmethod
    def empty(cls):
        cls.SESSIONS.empty()


class FileSession(DataSession):
    """
    Shelve based file system session engine that store the
    session information in a single file.

    This session engine should be used carefully as race
    conditions in the file access may corrupt its contents,
    thus making it not suitable for multi-threaded or multi-
    process environments.
    """

    SHELVE = None
    """ Global shelve object reference should reflect the
    result of opening a file in shelve mode, this is a global
    object and only one instance should exist per process """

    def __init__(self, name="file", *args, **kwargs):
        DataSession.__init__(self, name=name, *args, **kwargs)
        self["sid"] = self.sid

    @classmethod
    def new(cls, *args, **kwargs):
        if cls.SHELVE == None:
            cls.open()
        session = cls(*args, **kwargs)
        cls.SHELVE[session.sid] = session
        return session

    @classmethod
    def get_s(cls, sid, request=None):
        if cls.SHELVE == None:
            cls.open()
        session = cls.SHELVE.get(sid, None)
        if not session:
            return session
        is_expired = session.is_expired()
        if is_expired:
            cls.expire(sid)
        session = None if is_expired else session
        return session

    @classmethod
    def expire(cls, sid):
        del cls.SHELVE[sid]

    @classmethod
    def count(cls):
        return len(cls.SHELVE)

    @classmethod
    def all(cls):
        return cls.SHELVE

    @classmethod
    def open(cls, file_path="session.shelve"):
        base_path = config.conf("APPIER_BASE_PATH", "")
        base_path = config.conf("SESSION_FILE_PATH", base_path)
        if base_path and not os.path.exists(base_path):
            os.makedirs(base_path)
        base_path = os.path.expanduser(base_path)
        base_path = os.path.abspath(base_path)
        base_path = os.path.normpath(base_path)
        file_path = os.path.join(base_path, file_path)
        cls.SHELVE = shelve.open(file_path, protocol=2, writeback=True)
        cls.gc()

    @classmethod
    def close(cls):
        if not cls.SHELVE:
            return
        cls.SHELVE.close()
        cls.SHELVE = None

    @classmethod
    def empty(cls):
        for sid in cls.SHELVE:
            del cls.SHELVE[sid]

    @classmethod
    def gc(cls):
        for sid in cls.SHELVE:
            session = cls.SHELVE.get(sid, None)
            is_expired = session.is_expired()
            if is_expired:
                cls.expire(sid)

    @classmethod
    def db_type(cls):
        shelve_cls = type(cls.SHELVE.dict)
        shelve_dbm = shelve_cls.__name__
        return shelve_dbm

    @classmethod
    def db_secure(cls):
        return cls.db_type() == "dbm"

    def flush(self, request=None, secure=None):
        if not self.is_dirty():
            return
        self.mark(dirty=False)
        self.sync(secure=secure)

    def sync(self, secure=None):
        cls = self.__class__
        if secure == None:
            secure = cls.db_secure()
        if secure:
            cls.SHELVE.close()
            cls.open()
        else:
            cls.SHELVE.sync()


class RedisSession(DataSession):
    REDIS = None
    """ Global shelve object reference should reflect the
    result of opening a file in shelve mode, this is a global
    object and only one instance should exist per process """

    SERIALIZER = pickle
    """ The serializer to be used for the values contained in
    the session (used on top of the class) """

    def __init__(self, name="redis", *args, **kwargs):
        DataSession.__init__(self, name=name, *args, **kwargs)
        self["sid"] = self.sid

    @classmethod
    def new(cls, *args, **kwargs):
        if cls.REDIS == None:
            cls.open()
        session = cls(*args, **kwargs)
        data = cls.SERIALIZER.dumps(session)
        timeout = session.timeout()
        timeout = int(timeout)
        cls.REDIS.setex(session.sid, value=data, time=timeout)
        return session

    @classmethod
    def get_s(cls, sid, request=None):
        if cls.REDIS == None:
            cls.open()
        data = cls.REDIS.get(sid)
        if not data:
            return data
        session = cls.SERIALIZER.loads(data)
        is_expired = session.is_expired()
        if is_expired:
            cls.expire(sid)
        session = None if is_expired else session
        return session

    @classmethod
    def expire(cls, sid):
        cls.REDIS.delete(sid)

    @classmethod
    def count(cls):
        if cls.REDIS == None:
            cls.open()
        return cls.REDIS.dbsize()

    @classmethod
    def all(cls):
        if cls.REDIS == None:
            cls.open()
        sessions = dict()
        sids = cls.REDIS.keys()
        for sid in sids:
            data = cls.REDIS.get(sid)
            try:
                session = cls.SERIALIZER.loads(data)
            except Exception:
                continue
            sessions[sid] = session
        return sessions

    @classmethod
    def open(cls):
        cls.REDIS = redisdb.get_connection()

    @classmethod
    def empty(cls):
        if cls.REDIS == None:
            cls.open()
        cls.REDIS.flushdb()

    def flush(self, request=None):
        if not self.is_dirty():
            return
        self.mark(dirty=False)
        cls = self.__class__
        data = cls.SERIALIZER.dumps(self)
        timeout = self.timeout()
        timeout = int(timeout)
        cls.REDIS.setex(self.sid, value=data, time=timeout)


class ClientSession(DataSession):
    """
    Client side based session, that used encryption to store the
    complete set of information for the session on the client side
    through the usage of cookies.

    This session class provides extreme scalability as every server
    instance is independent and does not require inter-communication.
    """

    SERIALIZER = pickle
    """ The serializer to be used for the values contained in
    the session (used on top of the class) """

    COOKIE_LIMIT = 4096
    """ The limit (in bytes) of a cookie to be properly handled
    by most of the modern web browsers """

    def __init__(self, name="client", *args, **kwargs):
        DataSession.__init__(self, name=name, *args, **kwargs)
        self["sid"] = self.sid

    @classmethod
    def get_s(cls, sid, request=None):
        data_b64 = request.cookies.get("session", None)
        if not data_b64:
            return None
        data = base64.b64decode(data_b64)
        data = zlib.decompress(data)
        data = cls._verify(data, request)
        session = cls.SERIALIZER.loads(data)
        is_expired = session.is_expired()
        if is_expired:
            cls.expire(sid)
        session = None if is_expired else session
        return session

    @classmethod
    def _sign(cls, data, request):
        secret = cls._secret(request)
        secret = legacy.bytes(secret)
        digest = hmac.new(secret, data).hexdigest()
        digest = legacy.bytes(digest)
        data = digest + b":" + data
        return data

    @classmethod
    def _verify(cls, data, request):
        secret = cls._secret(request)
        secret = legacy.bytes(secret)
        digest, data = data.split(b":", 1)
        expected = hmac.new(secret, data).hexdigest()
        expected = legacy.bytes(expected)
        valid = digest == expected
        if not valid:
            raise exceptions.SecurityError(message="Invalid signature for message")
        return data

    @classmethod
    def _secret(cls, request):
        owner = request.owner
        if not hasattr(owner, "secret"):
            return None
        return owner.secret

    def flush(self, request=None):
        cls = self.__class__
        if not self.is_dirty():
            return
        self.mark(dirty=False)
        cls = self.__class__
        data = cls.SERIALIZER.dumps(self)
        data = cls._sign(data, request)
        data = zlib.compress(data)
        data_b64 = base64.b64encode(data)
        data_b64 = legacy.str(data_b64)
        valid = len(data_b64) <= cls.COOKIE_LIMIT
        if not valid:
            raise exceptions.OperationalError(
                message="Session payload size over cookie limit"
            )
        request.set_cookie = "session=%s" % data_b64



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import unittest

import appier


class BaseTest(unittest.TestCase):
    def setUp(self):
        self.app = appier.App()

    def tearDown(self):
        self.app.unload()

    def test_locale(self):
        self.app.locales = ("en_us", "pt_pt", "es_es")
        self.app._register_bundle(dict(hello="Hello"), "en_us")
        self.app._register_bundle(dict(hello="Olá"), "pt_pt")
        self.app._register_bundle(dict(hello="Bonjour"), "fr")

        result = self.app.to_locale("hello")
        self.assertEqual(result, "Hello")

        result = self.app.to_locale("hello", locale="en_us")
        self.assertEqual(result, "Hello")

        result = self.app.to_locale("hello", locale="en-us")
        self.assertEqual(result, "Hello")

        result = self.app.to_locale("hello", locale="pt_pt")
        self.assertEqual(result, "Olá")

        result = self.app.to_locale("hello", locale="pt-pt")
        self.assertNotEqual(result, "Olá")
        self.assertEqual(result, "Hello")

        result = self.app.to_locale("hello", locale="fr_fr")
        self.assertEqual(result, "Bonjour")

        result = self.app.to_locale("hello", locale="fr")
        self.assertEqual(result, "Bonjour")

        result = self.app.to_locale("hello", locale="es_es")
        self.assertNotEqual(result, "Hola")
        self.assertEqual(result, "Hello")

        result = self.app.has_locale("hello", locale="pt_pt")
        self.assertEqual(result, True)

        result = self.app.has_locale("Hello", locale="pt_pt")
        self.assertEqual(result, False)

        result = self.app.has_locale("hello", locale="fr_fr")
        self.assertEqual(result, True)

        result = self.app.has_locale("hello", locale="fr")
        self.assertEqual(result, True)

        result = self.app.has_locale("Hello", locale="fr")
        self.assertEqual(result, False)

        result = self.app.has_locale("hello", locale="es_es")
        self.assertEqual(result, False)

        result = self.app.has_locale("Hello", locale="es_es")
        self.assertEqual(result, False)

        self.app._register_bundle(dict(hello="Hola"), "es_es")

        result = self.app.to_locale("hello", locale="es_es")
        self.assertEqual(result, "Hola")

        result = self.app.to_locale("hello", locale="en")
        self.assertEqual(result, "Hello")

        result = self.app.to_locale("hello", locale="pt")
        self.assertEqual(result, "Olá")

        result = self.app.to_locale("hello", locale="es")
        self.assertEqual(result, "Hola")

        result = self.app.to_locale("bye")
        self.assertEqual(result, "bye")

        result = self.app.to_locale("bye", locale="cn")
        self.assertEqual(result, "bye")

        self.app._register_bundle(dict(bye="Bye"), "en_us")

        result = self.app.to_locale("bye")
        self.assertEqual(result, "Bye")

        result = self.app.to_locale("bye", locale="en_us")
        self.assertEqual(result, "Bye")

        result = self.app.to_locale("bye", locale="pt_pt")
        self.assertEqual(result, "Bye")

        result = self.app.to_locale("bye", locale="pt_pt", fallback=False)
        self.assertEqual(result, "bye")

        result = self.app.has_locale("bye", locale="en_us")
        self.assertEqual(result, True)

        result = self.app.has_locale("Bye", locale="en_us")
        self.assertEqual(result, False)

        result = self.app.has_locale("bye", locale="pt_pt")
        self.assertEqual(result, False)

        result = self.app.has_locale("Bye", locale="pt_pt")
        self.assertEqual(result, False)

    def test_locale_context(self):
        self.app.locales = ("en_us", "pt_pt", "es_es")
        self.app._register_bundle(dict(hello="Hello"), "en_us", context="extra")

        result = self.app.to_locale("hello")
        self.assertEqual(result, "Hello")

        result = self.app.to_locale("hello", context="extra")
        self.assertEqual(result, "Hello")

        result = self.app.to_locale("hello", context="other")
        self.assertEqual(result, "hello")

        result = self.app.has_locale("hello", context="extra")
        self.assertEqual(result, True)

        result = self.app.has_locale("hello", context="other")
        self.assertEqual(result, False)

        self.app._unregister_bundle(dict(hello="Hello"), "en_us", context="extra")

        result = self.app.has_locale("hello", context="extra")
        self.assertEqual(result, False)

    def test_field(self):
        request = appier.Request("GET", "/")
        request.set_params(
            dict(
                name=["john doe"],
                message=[""],
                valid_email=["john@doe.com"],
                invalid_email=["john"],
                valid_length=["1234"],
                invalid_length=["12345"],
            ),
        )
        self.app._request = request

        value = self.app.field("name")
        self.assertEqual(value, "john doe")

        value = self.app.field("message", mandatory=True)
        self.assertEqual(value, "")

        value = self.app.field(
            "valid_email", mandatory=True, not_empty=True, validation=(appier.is_email,)
        )
        self.assertEqual(value, "john@doe.com")

        value = self.app.field(
            "valid_length",
            mandatory=True,
            not_empty=True,
            validation=((appier.string_lt, 5),),
        )
        self.assertEqual(value, "1234")

        self.assertRaises(
            appier.OperationalError, lambda: self.app.field("other", mandatory=True)
        )

        self.assertRaises(
            appier.OperationalError,
            lambda: self.app.field("message", mandatory=True, not_empty=True),
        )

        self.assertRaises(
            appier.ValidationInternalError,
            lambda: self.app.field(
                "invalid_email",
                mandatory=True,
                not_empty=True,
                validation=(appier.is_email,),
            ),
        )

        self.assertRaises(
            appier.ValidationInternalError,
            lambda: self.app.field(
                "invalid_length",
                mandatory=True,
                not_empty=True,
                validation=((appier.string_lt, 5),),
            ),
        )

    def test_slugify(self):
        result = self.app.slugify("hello world")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "hello-world")

        result = self.app.slugify("olá mundo")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "ola-mundo")

    def test_pyslugify(self):
        if not self.app.pyslugify:
            if not hasattr(self, "skipTest"):
                return
            self.skipTest("No python-slugify engine present")

        result = self.app.slugify_pyslugify("hello world")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "hello-world")

        result = self.app.slugify_pyslugify("olá mundo")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "ola-mundo")

        result = self.app.slugify_pyslugify("你好世界")
        self.assertEqual(result, "ni-hao-shi-jie")

        result = self.app.slugify_pyslugify(
            appier.legacy.bytes("你好世界", encoding="utf-8")
        )
        self.assertEqual(type(result), str)
        self.assertEqual(result, "ni-hao-shi-jie")

    def test_slugier(self):
        if not self.app.slugier:
            if not hasattr(self, "skipTest"):
                return
            self.skipTest("No slugier engine present")

        result = self.app.slugify_slugier("hello world")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "hello-world")

        result = self.app.slugify_slugier("olá mundo")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "ola-mundo")

        result = self.app.slugify_slugier("你好世界")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "%e4%bd%a0%e5%a5%bd%e4%b8%96%e7%95%8c")

        result = self.app.slugify_slugier(
            appier.legacy.bytes("你好世界", encoding="utf-8")
        )
        self.assertEqual(type(result), str)
        self.assertEqual(result, "%e4%bd%a0%e5%a5%bd%e4%b8%96%e7%95%8c")

    def test_url_for(self):
        result = self.app.url_for("app.login")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "/login")

        result = self.app.url_for("app.login", query="query_string")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "/login?query_string")

        result = self.app.url_for("app.login", params=dict(query="query_string"))
        self.assertEqual(type(result), str)
        self.assertEqual(result, "/login?query=query_string")

        result = self.app.url_for("static", filename="hello.txt")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "/static/hello.txt")

        result = self.app.url_for("static", filename="hello.txt", compress="gzip")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "/static/hello.txt?compress=gzip")

    def test_filters(self):
        if not self.app.jinja:
            if not hasattr(self, "skipTest"):
                return
            self.skipTest("No Jinja2 template engine present")

        template = appier.Template("{{ message|locale }}")
        result = self.app.template(template, message="hello")
        self.assertEqual(result, "hello")

        template = appier.Template("{{ message|nl_to_br }}")
        result = self.app.template(template, message="hello\n")
        self.assertEqual(result, "hello<br/>\n")

        template = appier.Template("{{ message|sp_to_nbsp }}")
        result = self.app.template(template, message="hello world")
        self.assertEqual(result, "hello&nbsp;world")

    def test_unset_filter(self):
        if not self.app.jinja:
            if not hasattr(self, "skipTest"):
                return
            self.skipTest("No Jinja2 template engine present")

        template = appier.Template("{{ message|unset('world') }}")
        result = self.app.template(template)
        self.assertEqual(result, appier.legacy.u("world"))
        result = self.app.template(template, message="hello")
        self.assertEqual(result, appier.legacy.u("hello"))

        template = appier.Template("{{ message|unset(default = 'world') }}")
        result = self.app.template(template)
        self.assertEqual(result, appier.legacy.u("world"))
        result = self.app.template(template, message="hello")
        self.assertEqual(result, appier.legacy.u("hello"))

        template = appier.Template(
            "{{ message|unset(default = 'world', empty = True) }}"
        )
        result = self.app.template(template, message="")
        self.assertEqual(result, appier.legacy.u("world"))
        result = self.app.template(template, message="hello")
        self.assertEqual(result, appier.legacy.u("hello"))

        template = appier.Template(
            "{{ message|unset(default = 'world', extra = ('',)) }}"
        )
        result = self.app.template(template, message="")
        self.assertEqual(result, appier.legacy.u("world"))
        result = self.app.template(template, message="hello")
        self.assertEqual(result, appier.legacy.u("hello"))

        template = appier.Template(
            "{{ message|unset(default = 'world', extra = ('', 't')) }}"
        )
        result = self.app.template(template, message="t")
        self.assertEqual(result, appier.legacy.u("world"))
        result = self.app.template(template, message="hello")
        self.assertEqual(result, appier.legacy.u("hello"))

    def test_locale_filter(self):
        if not self.app.jinja:
            if not hasattr(self, "skipTest"):
                return
            self.skipTest("No Jinja2 template engine present")

        self.app._register_bundle(
            {
                "hello": appier.legacy.u("olá"),
                "world": appier.legacy.u("mundo"),
            },
            "pt_pt",
        )

        template = appier.Template("{{ message|locale }}")
        result = self.app.template(template, locale="pt_pt", message="hello")
        self.assertEqual(result, appier.legacy.u("olá"))
        result = self.app.template(template, locale="en_us", message="hello")
        self.assertEqual(result, appier.legacy.u("hello"))

        template = appier.Template("{{ 'hello'|locale }}")
        result = self.app.template(template, locale="pt_pt")
        self.assertEqual(result, appier.legacy.u("olá"))
        result = self.app.template(template, locale="en_us")
        self.assertEqual(result, appier.legacy.u("hello"))

    def test_css_abs(self):
        result = appier.base.CSS_ABS_REGEX.sub(
            b"url(http://www.example/\\2)", b"url(image.jpg)"
        )
        self.assertEqual(result, b"url(http://www.example/image.jpg)")

        result = appier.base.CSS_ABS_REGEX.sub(
            b"url(https://www.example/\\2)", b"url(image.jpg)"
        )
        self.assertEqual(result, b"url(https://www.example/image.jpg)")

        result = appier.base.CSS_ABS_REGEX.sub(
            b"url(http://www.example/\\2)", b"url(https://example.com/image.jpg)"
        )
        self.assertEqual(result, b"url(https://example.com/image.jpg)")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import os
import setuptools


def read_file(path):
    if not os.path.exists(path):
        return None
    file = open(path, "r")
    try:
        return file.read()
    finally:
        file.close()


setuptools.setup(
    name="appier",
    version="1.33.1",
    author="Hive Solutions Lda.",
    author_email="development@hive.pt",
    description="Appier Framework",
    license="Apache License, Version 2.0",
    keywords="appier framework web json wsgi",
    url="http://appier.hive.pt",
    zip_safe=False,
    packages=["appier", "appier.test"],
    test_suite="appier.test",
    package_dir={"": os.path.normpath("src")},
    package_data={
        "appier": [
            "res/static/css/*",
            "res/static/images/*",
            "res/static/js/*",
            "res/templates/*",
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    long_description=read_file("README.rst"),
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import io
import asyncio
import inspect
import tempfile

from . import util
from . import legacy
from . import exceptions


class ASGIApp(object):
    @classmethod
    async def asgi_entry(cls, scope, receive, send):
        if hasattr(cls, "_asgi") and cls._asgi:
            return await cls._asgi.app_asgi(scope, receive, send)
        cls._asgi = cls()
        return await cls._asgi.app_asgi(scope, receive, send)

    def serve_uvicorn(self, host, port, **kwargs):
        util.ensure_pip("uvicorn")
        import uvicorn

        self.server_version = uvicorn.__version__
        reload = kwargs.get("reload", False)
        forwarded_allow_ips = kwargs.get("forwarded_allow_ips", None)
        app_asgi = build_asgi_i(self)
        config = uvicorn.Config(
            app_asgi,
            host=host,
            port=port,
            reload=reload,
            forwarded_allow_ips=forwarded_allow_ips,
        )
        self._server = uvicorn.Server(config=config)
        self._server.run()

    def serve_hypercorn(
        self, host, port, ssl=False, key_file=None, cer_file=None, **kwargs
    ):
        util.ensure_pip("hypercorn")
        import hypercorn.config
        import hypercorn.asyncio

        self.server_version = hypercorn.__version__
        app_asgi = build_asgi_i(self)
        config = hypercorn.config.Config()
        config.bind = ["%s:%d" % (host, port)]
        config.keyfile = key_file if ssl else None
        config.certfile = cer_file if ssl else None
        server_coro = hypercorn.asyncio.serve(app_asgi, config)
        asyncio.run(server_coro)

    def serve_daphne(self, host, port, **kwargs):
        util.ensure_pip("daphne")
        import daphne.server
        import daphne.cli

        self.server_version = daphne.__version__
        app_asgi = build_asgi_i(self)
        app_daphne = daphne.cli.ASGI3Middleware(app_asgi)
        self._server = daphne.server.Server(
            app_daphne, endpoints=["tcp:port=%d:interface=%s" % (port, host)]
        )
        self._server.run()

    async def send(self, data, content_type=None):
        if content_type:
            self.response.set_content_type(content_type)
        return await self.response.send(data)

    async def app_asgi(self, *args, **kwargs):
        return await self.application_asgi(*args, **kwargs)

    async def application_asgi(self, scope, receive, send):
        """
        ASGI version of the application entrypoint, should define
        the proper asynchronous workflow for an HTTP request handling.

        :type scope: Dictionary
        :param scope: The connection scope, a dictionary that contains
        at least a type key specifying the protocol that is incoming.
        :type receive: Coroutine
        :param receive: An awaitable callable that will yield a new
        event dictionary when one is available.
        :type send: Coroutine
        :param send: an awaitable callable taking a single event dictionary
        as a positional argument that will return once the send has been
        completed or the connection has been closed.
        :see: https://asgi.readthedocs.io
        """

        scope_type = scope.get("type", None)
        scope_method = getattr(self, "asgi_" + scope_type, None)
        if not scope_method:
            raise exceptions.OperationalError(
                message="Unexpected scope type '%s'" % scope_type
            )

        return await scope_method(scope, receive, send)

    async def asgi_lifespan(self, scope, receive, send):
        running = True

        while running:
            event = await receive()

            if event["type"] == "lifespan.startup":
                self.start()
                await send(dict(type="lifespan.startup.complete"))

            elif event["type"] == "lifespan.shutdown":
                self.stop()
                await send(dict(type="lifespan.shutdown.complete"))
                running = False

    async def asgi_http(self, scope, receive, send):
        try:
            # sets the initials body value, to be replaced by the "real"
            # body file instance once it's created
            body = None

            # creates the context dictionary so that this new "pseudo" request
            # can have its own context for futures placement
            ctx = dict(start_task=None, encoding="utf-8")

            # runs the asynchronous building of the intermediate structures
            # to get to the final WSGI compliant environment dictionary
            start_response = await self._build_start_response(ctx, send)
            sender = await self._build_sender(ctx, send, start_response)
            body = await self._build_body(receive)
            environ = await self._build_environ(scope, body, sender)

            self.prepare()
            try:
                result = self.application_l(environ, start_response, ensure_gen=False)
                self.set_request_ctx()
            finally:
                self.restore()

            # verifies if the resulting value is an awaitable and if
            # that's the case waits for it's "real" result value (async)
            if inspect.isawaitable(result):
                result = await result  # @UndefinedVariable

            # waits for the start (code and headers) send operation to be
            # completed (async) so that we can proceed with body sending
            self._ensure_start(ctx, start_response)
            await ctx["start_task"]

            # iterates over the complete set of chunks in the response
            # iterator to send each of them to the client side
            for chunk in result if result else []:
                if asyncio.iscoroutine(chunk):
                    await chunk
                elif asyncio.isfuture(chunk):
                    await chunk
                elif isinstance(chunk, int):
                    continue
                else:
                    if legacy.is_string(chunk):
                        chunk = chunk.encode(ctx["encoding"])
                    await send(
                        {"type": "http.response.body", "body": chunk, "more_body": True}
                    )

            # sends the final empty chunk indicating the end
            # of the body payload to the "owning" server
            await send({"type": "http.response.body", "body": b""})
        finally:
            if body:
                body.close()
            self.unset_request_ctx()

    async def _build_start_response(self, ctx, send):
        def start_response(status, headers):
            if ctx["start_task"]:
                return
            code = status.split(" ", 1)[0]
            code = int(code)
            headers = [
                (name.lower().encode("ascii"), value.encode("ascii"))
                for name, value in headers
            ]
            send_coro = send(
                {"type": "http.response.start", "status": code, "headers": headers}
            )
            ctx["start_task"] = asyncio.create_task(send_coro)

        return start_response

    async def _build_sender(self, ctx, send, start_response):
        async def sender(data):
            self._ensure_start(ctx, start_response)
            await ctx["start_task"]
            if legacy.is_string(data):
                data = data.encode(ctx["encoding"])
            return await send(
                {"type": "http.response.body", "body": data, "more_body": True}
            )

        return sender

    async def _build_body(self, receive, max_size=65536):
        body = tempfile.SpooledTemporaryFile(max_size=max_size)
        while True:
            message = await receive()
            util.verify(message["type"] == "http.request")
            body.write(message.get("body", b""))
            if not message.get("more_body"):
                break
        body.seek(0)
        return body

    async def _build_environ(self, scope, body, sender):
        """
        Builds a scope and request body into a WSGI environ object.

        :type scope: Dictionary
        :param scope: The scope dictionary from ASGI.
        :type: body: File
        :param body: The body callable to be used for the reading
        of the input.
        :type sender: Function
        :param sender: The sender function responsible for the sending
        of data to the client side (response).
        :rtype: Dictionary
        :return: The WSGI compatible environ dictionary converted
        from ASGI and ready to be used by WSGI apps.
        """

        environ = {
            "REQUEST_METHOD": scope["method"],
            "SCRIPT_NAME": scope.get("root_path", ""),
            "PATH_INFO": scope["path"],
            "QUERY_STRING": scope["query_string"].decode("ascii"),
            "SERVER_PROTOCOL": "HTTP/%s" % scope["http_version"],
            "wsgi.version": (1, 0),
            "wsgi.url_scheme": scope.get("scheme", "http"),
            "wsgi.input": body,
            "wsgi.output": sender,
            "wsgi.errors": io.BytesIO(),
            "wsgi.multithread": True,
            "wsgi.multiprocess": True,
            "wsgi.run_once": False,
        }

        if "server" in scope:
            environ["SERVER_NAME"] = scope["server"][0]
            environ["SERVER_PORT"] = str(scope["server"][1])
        else:
            environ["SERVER_NAME"] = "localhost"
            environ["SERVER_PORT"] = "80"

        if "client" in scope:
            environ["REMOTE_ADDR"] = scope["client"][0]

        for name, value in scope.get("headers", []):
            name = name.decode("latin1")
            if name == "content-length":
                corrected_name = "CONTENT_LENGTH"
            elif name == "content-type":
                corrected_name = "CONTENT_TYPE"
            else:
                corrected_name = "HTTP_%s" % name.upper().replace("-", "_")

            value = value.decode("latin1")
            if corrected_name in environ:
                value = environ[corrected_name] + "," + value
            environ[corrected_name] = value

        return environ

    def _ensure_start(self, ctx, start_response):
        if ctx["start_task"]:
            return
        self.request_ctx.set_headers_b()
        code_s = self.request_ctx.get_code_s()
        headers = self.request_ctx.get_headers() or []
        if self.sort_headers:
            headers.sort()
        start_response(code_s, headers)


def build_asgi(app_cls):
    async def app_asgi(scope, receive, send):
        return await app_cls.asgi_entry(scope, receive, send)

    return app_asgi


def build_asgi_i(app):
    async def app_asgi(scope, receive, send):
        return await app.app_asgi(scope, receive, send)

    return app_asgi



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import unittest

import appier


class ValidationTest(unittest.TestCase):

    def test_is_simple(self):
        self.assertEqual(appier.is_simple("value")(dict(value=""), None), True)
        self.assertEqual(appier.is_simple("value")(dict(value="hello"), None), True)
        self.assertEqual(
            appier.is_simple("value")(dict(value="hello world"), None), True
        )

        with self.assertRaises(appier.ValidationInternalError):
            self.assertEqual(
                appier.is_simple("value")(dict(value="hello?world"), None), True
            )
        with self.assertRaises(appier.ValidationInternalError):
            self.assertEqual(
                appier.is_simple("value")(dict(value="hello=world"), None), True
            )
        with self.assertRaises(appier.ValidationInternalError):
            self.assertEqual(
                appier.is_simple("value")(
                    dict(value=appier.legacy.u("你好世界")), None
                ),
                True,
            )

    def test_is_email(self):
        self.assertEqual(appier.is_email("value")(dict(value=""), None), True)
        self.assertEqual(
            appier.is_email("value")(dict(value="user@domain.com"), None), True
        )
        self.assertEqual(
            appier.is_email("value")(dict(value="first.second@domain.com"), None), True
        )
        self.assertEqual(
            appier.is_email("value")(dict(value="first+second@domain.com"), None), True
        )
        self.assertEqual(
            appier.is_email("value")(
                dict(value=appier.legacy.u("你好世界@domain.com")), None
            ),
            True,
        )

        with self.assertRaises(appier.ValidationInternalError):
            appier.is_email("value")(dict(value="user"), None)
        with self.assertRaises(appier.ValidationInternalError):
            appier.is_email("value")(dict(value="domain"), None)
        with self.assertRaises(appier.ValidationInternalError):
            appier.is_email("value")(dict(value="domain.com"), None)
        with self.assertRaises(appier.ValidationInternalError):
            appier.is_email("value")(dict(value="user@"), None)
        with self.assertRaises(appier.ValidationInternalError):
            appier.is_email("value")(dict(value=appier.legacy.u("你好世界")), None)
        with self.assertRaises(appier.ValidationInternalError):
            appier.is_email("value")(dict(value=appier.legacy.u("你好世界@")), None)

    def test_is_url(self):
        self.assertEqual(appier.is_url("value")(dict(value=""), None), True)
        self.assertEqual(
            appier.is_url("value")(dict(value="http://hello.com"), None), True
        )
        self.assertEqual(
            appier.is_url("value")(dict(value="http://hello.world"), None), True
        )
        self.assertEqual(
            appier.is_url("value")(dict(value="https://hello.world"), None), True
        )
        self.assertEqual(
            appier.is_url("value")(
                dict(value=appier.legacy.u("https://你好世界")), None
            ),
            True,
        )
        self.assertEqual(appier.is_url("value")(dict(value="http://hello"), None), True)
        self.assertEqual(
            appier.is_url("value")(dict(value="mailto://hello"), None), True
        )

        with self.assertRaises(appier.ValidationInternalError):
            appier.is_email("value")(dict(value=appier.legacy.u("http:")), None)
        with self.assertRaises(appier.ValidationInternalError):
            appier.is_email("value")(dict(value=appier.legacy.u("http://")), None)
        with self.assertRaises(appier.ValidationInternalError):
            appier.is_email("value")(dict(value=appier.legacy.u("mailto://")), None)
